package br.usp.ime.coed.DEFSupport;




/**
 * This class is responsible for how the drawing elements will be
 * treated through the program execution.<BR>
 * To make this possible, a DrawingElementFinder object manages
 * the view conduct with the help of a DEFBehaviour object.
 * @see DEFBehaviour
 */

public class DrawingElementFinder{

  /**
   * This is the view.
   */
  private AbstractDrawingElementFinderClient client;
  /**
   * This represents the behaviour.
   */
  private DEFBehaviour behaviour;

  /**
   * This constructor sets the view of this finder, and its behaviour.
   * Its behaviour is automatically added to the view list of listeners.
   * @param view is the object listened to.
   * @param behaviour represents the view listener behaviour.
   */
  public DrawingElementFinder(AbstractDrawingElementFinderClient client,
         DEFBehaviour behaviour) {
        this. client = client;
        this.behaviour = behaviour;
        client.addMouseListener(behaviour);
        client.addMouseMotionListener(behaviour);
  }

   /**
   * This constructor sets the view of this finder.
   * Its behaviour is set as a DefaultDEFBehaviour object.
   * This is automatically added to the view list of listeners.
   * @param view is the object listened to.
   */
  public DrawingElementFinder(AbstractDrawingElementFinderClient client) {
        this(client, new DefaultDEFBehaviour( client ));
  }

  /**
   * Sets a new behaviour to this finder.
   * @param behaviour the new behaviour of this finder
   */
  public void setDEFBehaviour(DEFBehaviour behaviour, boolean clearSelection){
    
    client.removeMouseListener(this.behaviour);
    client.removeMouseMotionListener(this.behaviour);
    if(clearSelection)
	client.clearSelection();

    client.addMouseListener(behaviour);
    client.addMouseMotionListener(behaviour);

    this.behaviour = behaviour;


  }

} //  class DrawingElementFinder







