package br.usp.ime.coed.behaviour;

import java.awt.event.MouseEvent;

import br.usp.ime.coed.ProgramColors;
import br.usp.ime.coed.DEFSupport.DEFBehaviour;
import br.usp.ime.coed.model.ApplicationModel;
import br.usp.ime.coed.model.CoedDocumentCareTaker;
import br.usp.ime.coed.view.ApplicationView;
import br.usp.ime.coed.view.GraphicPane;
import br.usp.ime.coed.view.GraphicalIconElement;
import br.usp.ime.coed.view.Pipe;
import br.usp.ime.coed.view.PipeFormat;

/**
 * The creating pipe behaviour. It is activated when the user requests a new
 */
public class CreatePipeBehaviour implements DEFBehaviour {

    ApplicationView view;
    
    ApplicationModel model;

    GraphicPane pane;

    GraphicalIconElement from;

    GraphicalIconElement to;

    Pipe tempPipe;

    /**
     * @param view
     * @param pane
     */
    public CreatePipeBehaviour(ApplicationView view, GraphicPane pane) {
        this.view = view;
        this.pane = pane;
        this.model = view.getApplicationModel();
        pane.getHighlighter().setHighlight(
                PipeFormat.getAnimationFormat().getLineColor(), 4.0F);
        view.setAllActionEnable(false);
        
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
     */

    public void mouseClicked(MouseEvent e) {
   
        if (e.getButton() == MouseEvent.BUTTON1) {

            GraphicalIconElement pointedElement;
            try {
                pointedElement = (GraphicalIconElement) pane.getContainer()
                        .searchElementAt(e.getPoint());
            } catch (ClassCastException cce) {
                return;
            }catch(NullPointerException npe){
                return;
            }

            if (from == null  && pointedElement.getComponent().getNextComponents().size() > 0) {
                return;
            }
            if (from != null && to == null && pointedElement.getComponent().getPreviousComponents().size() > 0) {
                return;
            }
            
            
            pane.filledSelection(ProgramColors.getSelectionColor());
            if (from == null) {
                from = pointedElement;
                pane.drawingElementClicked(from, e);
                return;
            } else if ((to == null) && (to != from)) {
                to = pointedElement;
                Pipe pipe = new Pipe(from, to, PipeFormat.getDefaultFormat());
                CoedDocumentCareTaker caretaker = model.getDocManager().getDocumentCareTaker(pane.getDocument());
                caretaker.addMemento(pane.getDocument().createMemento());
                pane.getDocument().addPipe(from.getComponent(),to.getComponent());
                finish(e);
            }

        }else{
            finish(e);
        }
 
    }
    
    private void finish(MouseEvent e){
        from = null;
        to = null;
        pane.getDrawingElementFinder().setDEFBehaviour(
                new CoedDefaultBehaviour(view), true);
        pane.drawingElementClicked(null, e);
        pane.dashedSelection();
        pane.setTemporaryPipe(null);
        view.setStatus(" ");
        view.setAllActionEnable(true);
        view.getApplicationModel().getDocManager().updateObserver();

    }

    /*
     * (non-Javadoc)
     * 
     * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
     */
    public void mouseEntered(MouseEvent e) {
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
     */
    public void mouseExited(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
     */
    public void mousePressed(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
     */
    public void mouseReleased(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
     */
    public void mouseDragged(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
     */
    public void mouseMoved(MouseEvent e) {
        GraphicalIconElement pointedElement;
        try {
            pointedElement = (GraphicalIconElement) pane.getContainer()
                    .searchElementAt(e.getPoint());
        } catch (ClassCastException cce) {
            return;
        }


        if (tempPipe != null)
            pane.setTemporaryPipe(null);

        if (from != null && 
                (pointedElement != null && 
                 from != pointedElement && 
                 pointedElement.getComponent().getPreviousComponents().size()<=0) && 
                 !from.getComponent().containsComponentInSamePipe(pointedElement.getComponent())) {
    
            
            pane.drawingElementFliedOver(pointedElement);
            tempPipe = new Pipe(from, pointedElement, PipeFormat
                    .getAnimationFormat());
            pane.setTemporaryPipe(tempPipe);
        } else if (from == null) {
            pane.drawingElementFliedOver(pointedElement);
        } else {
            pane.drawingElementFliedOver(null);
        }

    }

}