package br.usp.ime.coed.model;

import java.util.ArrayList;
import java.util.List;

import javax.swing.ImageIcon;

import br.usp.ime.coed.ReadEgeneConfiguration;
import br.usp.ime.coed.Main;

/**
 * The application model.
 * 
 * @author Andr Yoshiaki Kashiwabara <yoshiaki@vision.ime.usp.br>
 * @see br.usp.ime.coed.view.ApplicationView
 */
public class ApplicationModel {

    /**
     * The application name
     */
    private String name = "EGene configuration editor";

    /** All components type */
    private List componentsType;

    /** The application icon */
    private ImageIcon icon;

    /**
     * The document manager instance
     */
    private CoedDocumentManager docManager;

    /**
     * The transfer area instance
     */
    private TransferAreaManager transferAreaManager;

    /**
     * The default constructor
     */
    public ApplicationModel() {
        docManager = new CoedDocumentManager();
        componentsType = new ArrayList();
        ReadEgeneConfiguration rec = ReadEgeneConfiguration.getInstance();
        componentsType = rec.iconList();
        icon = new ImageIcon(Main.getInstance().getEgeneDir() + "/images/e.gif");
        transferAreaManager = new TransferAreaManager();
    }

    /**
     * Get the application icon
     * 
     * @return <code>ImageIcon</code> instance of the application icon
     */
    public ImageIcon getIcon() {
        return icon;
    }

    /**
     * Set the application icon
     * 
     * @param icon
     *            is the new icon that will be used
     */
    public void setIcon(ImageIcon icon) {
        this.icon = icon;
    }

    /**
     * Get the application name
     * 
     * @return <code>String</code> instance of the application name
     */
    public String getName() {
        return name;
    }

    /**
     * Set the application name
     * 
     * @param name
     *            is the new name that will be used
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Returns a list of the all component types
     * 
     * @return <code>List</code> of component types
     */
    public List getComponentsType() {
        return componentsType;
    }

    /**
     * Set the component type list
     * 
     * @param componentsType
     *            the new list
     */
    public void setComponentsType(List componentsType) {
        this.componentsType = componentsType;
    }

    /**
     * Returns the CoedDocumentManager
     * 
     * @see br.usp.ime.model.CoedDocumentManager
     * @return Returns the docManager.
     */
    public CoedDocumentManager getDocManager() {
        return docManager;
    }

    /**
     * Set a new component manager
     * 
     * @param Returns
     *            the docManager
     */
    public void setDocManager(CoedDocumentManager docManager) {
        this.docManager = docManager;
    }

    /**
     * Returns the transfer area
     * 
     * @see br.usp.ime.coed.mode.TransferArea
     * @return <code>TransferAreaManager</code>
     */
    public TransferAreaManager getTransferAreaManager() {
        return transferAreaManager;
    }

    /**
     * Set a new transfer area
     * 
     * @see br.usp.ime.coed.mode.TransferArea
     * @param transferAreaManager
     */
    public void setTransferAreaManager(TransferAreaManager transferAreaManager) {
        this.transferAreaManager = transferAreaManager;
    }

}

