/*
 * Criado em 23/10/2004
 */
package br.usp.ime.coed.model;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

import br.usp.ime.coed.ReadEgeneConfiguration;

/**
 * A Memento implemention for CoedDocument
 * 
 * For more details, look to Memento pattern.
 * 
 * @see br.usp.ime.coed.CoedDocument
 * @see br.usp.ime.coed.CoedDocumentCareTaker
 * 
 * @author Andr Yoshiaki Kashiwabara <yoshiaki@vision.ime.usp.br>
 */
public class CoedDocumentMemento {
//TODO: Precisa ser refatorado, contm trechos copy-paste e mtodos gigantescos !!!
    /**
     * The state
     */
    private CoedDocument document;

    /**
     * Returns a CoedDocument state
     * 
     * @return Returns a CoedDocument state
     */
    public CoedDocument getState() {
        return document;
    }

    /**
     * Creates a snapshot of the document
     * 
     * @param orig
     *            is the original document
     */
    public void setState(CoedDocument orig) {
        // A state must be a copy of the object.
        HashMap origToNew = new HashMap();
        document = new CoedDocument(orig.getName());
        Iterator it = orig.getAllComponents().iterator();
        HashSet first = new HashSet();
        document.setPipelineName(orig.getPipelineName());
        document.setNotSaved(orig.modified());
        // Add all components
        while (it.hasNext()) {
            Component origC = (Component) it.next();
            Component c = new Component(origC.getType());
            c.setPhase(new String(origC.getPhase()));
            c.setX(origC.getX());
            c.setY(origC.getY());
            document.addComponent(c);
            origToNew.put(origC, c);
            HashMap mandatory = (HashMap) origC.getMandatoryValues();
            HashMap newM = new HashMap();
            Iterator it3 = mandatory.keySet().iterator();
            while (it3.hasNext()) {
                String key = (String) it3.next();
                newM.put(key, mandatory.get(key));
            }
            HashMap optional = (HashMap) origC.getOptionalValues();
            HashMap newOp = new HashMap();
            it3 = optional.keySet().iterator();
            while (it3.hasNext()) {
                String key = (String) it3.next();
                newOp.put(key, optional.get(key));
            }
            c.setMandatoryValues(newM);
            c.setOptionalValues(newOp);
        }

        // Add all pipes
        it = origToNew.keySet().iterator();
        while (it.hasNext()) {
            Component origC = (Component) it.next();
            Iterator it2 = origC.getNextComponents().iterator();
            while (it2.hasNext()) {
                document.addPipe((Component) origToNew.get(origC),
                        (Component) origToNew.get(it2.next()));

            }
        }
        document.setFirstComponent((Component) origToNew.get(orig
                .getFirstComponent()));

    }

    /**
     * Set the state from a EGene configuration file
     * 
     * @param egeneFileString -
     *            a egene configuration file
     */
    public void setStateFromEGeneFile(String egeneFileString) {
        document = new CoedDocument();
        String[] lines = egeneFileString.split("\n");
        boolean readingPhaseHeader = false;
        Component component = null;
        Map mandatory = null;
        Map optional = null;
        double x = 10.0;
        double y = 50.0;
        LinkedList components = new LinkedList();
        boolean readingParameters = false;

        for (int i = 0; i < lines.length; i++) {
            String line = lines[i];
            line = line.trim();
            if (line.matches("#.*") || line.matches("\\s*")) {
                continue;
            }

            if (line.matches("PIPELINE.*")) {
                document.setName(line.replaceFirst("PIPELINE\\s*=\\s*", ""));
                document.setPipelineName(document.getName());
                readingPhaseHeader = true;
                continue;
            }

            if (line.matches("PHASE.*")) {
                component = new Component(null);
                document.addComponent(component);
                components.add(component);
                component.setX(x);
                component.setY(y);
                component.setPhase(line.replaceAll("PHASE\\s*=\\s*", ""));
                x += 120;
                if (x > 700) {
                    x = 10.0;
                    y += 120.0;
                }
                readingPhaseHeader = true;
                readingParameters = false;
            }
            if (readingPhaseHeader && line.matches("program.*")) {
                String program = line.replaceFirst("program\\s+=\\s+", "");
                program = program.replaceAll("\\.pl", "");
                ComponentType type = ReadEgeneConfiguration.getInstance()
                        .iconList(program);
                component.setType(type);
                mandatory = ReadEgeneConfiguration.getInstance()
                        .getMandatoryParameters(program);
                optional = ReadEgeneConfiguration.getInstance()
                        .getOptionalParameters(program);
                readingPhaseHeader = false;
                readingParameters = true;
                continue;
            } else if (!readingPhaseHeader && readingParameters) {
                if (!line.matches(".*=.*")) {
                    continue;
                }
                String[] option = line.split("\\s*=\\s*");
                option[0] = option[0].replaceAll("\\s+", "");
                if (mandatory.containsKey(option[0])) {
                    component.getMandatoryValues().put(new String(option[0]),
                            new String(option[1]));
                } else if (optional.containsKey(option[0])) {
                    component.getOptionalValues().put(new String(option[0]),
                            new String(option[1]));
                }
            }
        }
        Iterator it = components.iterator();
        Component ant = null;
        if (it.hasNext()) {
            ant = (Component) it.next();
            document.setFirstComponent(ant);
        }
        while (it.hasNext()) {
            Component c = (Component) it.next();
            document.addPipe(ant, c);
            ant = c;
        }

    }

    /**
     * Set the state from the XML file.
     * 
     * @param xml
     *            the xml string
     */
    public void setState(String xml) {
        Document doc = null;
        document = new CoedDocument();
        try {
            HashMap idToType = new HashMap();
            HashMap idToComponent = new HashMap();
            HashMap pipes = new HashMap();
            doc = DocumentHelper.parseText(xml);
            Iterator it = doc.getRootElement().elementIterator();
            String firstElementId = doc.getRootElement().element(
                    "first_component").getStringValue();
            String pipelineName = doc.getRootElement().element("pipeline_name")
                    .getStringValue();
            document.setPipelineName(pipelineName);
            while (it.hasNext()) {
                Element el = (Element) it.next();
                if (el.getName().equals("component_type")) {
                    String type = "";
                    String id = "";
                    Iterator it2 = el.elementIterator();
                    while (it2.hasNext()) {
                        Element el2 = (Element) it2.next();
                        if (el2.getName().equals("name")) {
                            type = el2.getStringValue();
                        } else if (el2.getName().equals("id")) {
                            id = el2.getStringValue();
                            idToType.put(id, type);
                        }
                    }
                } else if (el.getName().equals("component")) {
                    Component c = new Component(null);
                    HashMap mandatory = new HashMap();
                    HashMap optional = new HashMap();
                    c
                            .setX(Double.parseDouble(el.element("x")
                                    .getStringValue()));
                    c
                            .setY(Double.parseDouble(el.element("y")
                                    .getStringValue()));
                    Iterator it2 = el.elementIterator();
                    while (it2.hasNext()) {
                        Element el2 = (Element) it2.next();
                        if (el2.getName().equals("name")) {
                            c.setPhase(el2.getStringValue());
                        } else if (el2.getName().equals("mandatory_parameter")) {
                            Element name = el2.element("name");
                            Element value = el2.element("value");
                            mandatory.put(name.getStringValue(), value
                                    .getStringValue());
                        } else if (el2.getName().equals("optional_parameter")) {
                            Element name = el2.element("name");
                            Element value = el2.element("value");
                            optional.put(name.getStringValue(), value
                                    .getStringValue());
                        } else if (el2.getName().equals("component_id")) {
                            idToComponent.put(el2.getStringValue(), c);
                            if (!firstElementId.equals("")
                                    && el2.getStringValue().equals(
                                            firstElementId)) {
                                document.setFirstComponent(c);
                            }
                        } else if (el2.getName().equals("component_type_id")) {
                            ComponentType t = ReadEgeneConfiguration
                                    .getInstance().iconList(
                                            idToType.get(el2.getStringValue()));
                            c.setType(t);
                        } else if (el2.getName().equals("next")) {
                            pipes.put(c, el2.getStringValue());
                        }

                    }
                    c.setMandatoryValues(mandatory);
                    c.setOptionalValues(optional);
                    document.addComponent(c);
                    Iterator it3 = pipes.keySet().iterator();
                    while (it3.hasNext()) {
                        Component c1 = (Component) it3.next();
                        Component c2 = (Component) idToComponent.get(pipes
                                .get(c1));
                        if (c2 != null)
                            document.addPipe(c1, c2);
                    }

                }
            }
        } catch (DocumentException e) {
            e.printStackTrace();
        }

    }

    /**
     * Returns the xml string representing the state
     * 
     * @return xml string
     */
    public String xmlString() {
        HashMap typeToId = new HashMap();
        HashMap componentToId = new HashMap();
        StringBuffer out = new StringBuffer(); // Do not use "string += string "-- it is very slow.
        out.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
        out
                .append("<coed:coed xmlns:coed=\"http://www.ibm.com\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://www.lbm.fmvz.usp.br/xml/2004/coed.xsd \">\n");
        out.append("<name>").append(document.getName()).append("</name>\n");
        out.append("<pipeline_name>").append(document.getPipelineName())
                .append("</pipeline_name>");
        componentTypeToXML(out, typeToId);
        componentToXML(typeToId, componentToId, out);
        out.append("<first_component>").append(
                componentToId.get(document.getFirstComponent())).append(
                "</first_component>");
        out.append("</coed:coed>");
        return out.toString();
    }

    /**
     * Generate the components XML representation
     * 
     * @param typeToId
     * @param componentToId
     * @param id
     * @param out
     */
    private void componentToXML(HashMap typeToId, HashMap componentToId,
            StringBuffer out) {
        Iterator it1 = document.getAllComponents().iterator();
        int id = 1;
        while (it1.hasNext()) {
            Component c = (Component) it1.next();
            componentToId.put(c, new Integer(id));
            id++;
        }
        it1 = document.getAllComponents().iterator();
        while (it1.hasNext()) {
            Component c = (Component) it1.next();
            out.append("<component>\n");
            out.append("\t<name>").append(c.getPhase()).append("</name>\n");
            HashMap mandatory = (HashMap) c.getMandatoryValues();
            Iterator it2 = mandatory.keySet().iterator();
            while (it2.hasNext()) {
                String key = (String) it2.next();
                out.append("\t<mandatory_parameter>\n");
                out.append("\t\t<name>").append(key).append("</name>\n");
                out.append("\t\t<value>").append(mandatory.get(key)).append(
                        "</value>\n");
                out.append("\t</mandatory_parameter>\n");
            }
            HashMap optional = (HashMap) c.getOptionalValues();
            it2 = optional.keySet().iterator();
            while (it2.hasNext()) {
                String key = (String) it2.next();
                out.append("\t<optional_parameter>\n");
                out.append("\t\t<name>").append(key).append("</name>\n");
                out.append("\t\t<value>").append(optional.get(key)).append(
                        "</value>\n");
                out.append("\t</optional_parameter>\n");
            }

            out.append("\t<component_id>").append(componentToId.get(c)).append(
                    "</component_id>\n");
            it2 = c.getNextComponents().iterator();
            while (it2.hasNext()) {
                out.append("\t<next>").append(componentToId.get(it2.next()))
                        .append("</next>\n");
            }
            out.append("\t<component_type_id>").append(
                    typeToId.get(c.getType())).append("</component_type_id>\n");
            out.append("\t<x>").append(c.getX()).append("</x>\n");
            out.append("\t<y>").append(c.getY()).append("</y>\n");
            out.append("</component>\n");

        }
    }

    /**
     * The ComponentType list to XML
     * 
     * @param out
     * @return
     */
    private void componentTypeToXML(StringBuffer out, HashMap componentType) {
        int id = 1;
        Iterator it1 = ReadEgeneConfiguration.getInstance().iconList()
                .iterator();
        while (it1.hasNext()) {
            ComponentType type = (ComponentType) it1.next();
            out.append("<component_type>\n");
            out.append("\t<name>").append(type.getName()).append("</name>\n");
            HashMap mandatory = (HashMap) ReadEgeneConfiguration.getInstance()
                    .getMandatoryParameters(type.getName());
            Iterator it2 = mandatory.keySet().iterator();
            while (it2.hasNext()) {
                String key = (String) it2.next();
                out.append("\t<mandatory_parameter>\n");
                out.append("\t\t<name>").append(key).append("</name>\n");
                out.append("\t\t<value>").append(mandatory.get(key)).append(
                        "</value>\n");
                out.append("\t</mandatory_parameter>\n");
            }

            HashMap optional = (HashMap) ReadEgeneConfiguration.getInstance()
                    .getOptionalParameters(type.getName());
            it2 = optional.keySet().iterator();
            while (it2.hasNext()) {
                String key = (String) it2.next();
                out.append("\t<optional_parameter>\n");
                out.append("\t\t<name>").append(key).append("</name>\n");
                out.append("\t\t<value>").append(optional.get(key)).append(
                        "</value>\n");
                out.append("\t</optional_parameter>\n");
            }
            out.append("\t<icon>").append(type.abbreviature()).append(
                    "</icon>\n");
            out.append("\t<id>" + id + "</id>\n");
            out.append("</component_type>\n");
            componentType.put(type, new Integer(id));
            id += 1;

        }
    }
}