/*
 * Criado em 22/10/2004
 */
package br.usp.ime.coed.model;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Observable;
import java.util.Set;

/**
 * GraphicalIconElement (Model)
 * 
 * @author Andr Yoshiaki Kashiwabara <yoshiaki@vision.ime.usp.br>
 */
public class Component extends Observable {
    /** The phase */
    private String phase;

    /** The component type */
    private ComponentType type;

    /** Mandatory parameters */
    private Map mandatoryValues;

    /** opcional parameters */
    private Map optionalValues;

    /** x position */
    private double x;

    /** y position */
    private double y;

    /** next elements */
    public Set nextComponents;

    /** previous elements */
    public Set previousComponents;

    public static final String COMPONENT_MOVED = "COMPONENT_MOVED";

    public static final String COMPONENT_SELECTED = "COMPONENT_SELECTED";

    public static final String COMPONENT_DESELECTED = "COMPONENT_DESELECTED";

    /**
     * Default constructor
     */
    public Component(ComponentType type) {
        mandatoryValues = new HashMap();
        optionalValues = new HashMap();
        nextComponents = new HashSet();
        previousComponents = new HashSet();
        this.type = type;
        if (type != null)
            phase = type.getName();
    }

    /**
     * Overrides the superclass <code>Object</code> method
     * 
     * @return a <code>String</code> value
     */
    public String toString() {
        String string = "#====================================================================================\n";
        Iterator iteratorMandatory = mandatoryValues.keySet().iterator();
        Iterator iteratorOptional = optionalValues.keySet().iterator();
        string = string + "PHASE=" + phase + "\n";
        string = string + "program = " + type.getName() + ".pl" + "\n";
        if (iteratorOptional.hasNext() || iteratorMandatory.hasNext()) {
            string = string
                    + "#-----------------------------------------------------------------------------------\n";
        }

        while (iteratorMandatory.hasNext()) {
            String question = (String) iteratorMandatory.next();
            string = string + question + " = " + mandatoryValues.get(question)
                    + "\n";
        }

        while (iteratorOptional.hasNext()) {
            String question = (String) iteratorOptional.next();
            String optional = (String) optionalValues.get(question);
            if (!optional.equals("")) {
                string = string + question + " = " + optional + "\n";
            }
        }
        string = string
                + "#====================================================================================\n\n";
        return string;
    }

    public void addNextComponent(Component component) {
        nextComponents.add(component);
        setChanged();
    }

    public void removeNextComponent(Component component) {
        nextComponents.remove(component);
        setChanged();
    }

    public void addPreviousComponent(Component component) {
        previousComponents.add(component);
        setChanged();
    }

    public void removePreviousComponent(Component component) {
        previousComponents.remove(component);
        setChanged();
    }

    public Map getMandatoryValues() {
        return mandatoryValues;
    }

    public void setMandatoryValues(Map mandatoryParameters) {
        this.mandatoryValues = mandatoryParameters;
        setChanged();
    }

    public Map getOptionalValues() {
        return optionalValues;
    }

    public void setOptionalValues(Map optionalParameters) {
        this.optionalValues = optionalParameters;
        setChanged();
    }

    public ComponentType getType() {
        return type;
    }

    public void setType(ComponentType type) {
        this.type = type;
        setChanged();
    }

    public double getX() {
        return x;
    }

    public void setX(double x) {
        this.x = x;
        setChanged();
    }

    public double getY() {
        return y;

    }

    public void setY(double y) {
        this.y = y;
        setChanged();
    }

    public String getPhase() {
        return phase;
    }

    public void setPhase(String phase) {
        this.phase = phase;
        setChanged();
    }

    /**
     * @param x2
     * @param y2
     */
    public void setPosition(double x2, double y2) {
        this.x = x2;
        this.y = y2;
        setChanged();
        notifyObservers(COMPONENT_MOVED);
    }

    public Set getNextComponents() {
        return nextComponents;
    }

    public Set getPreviousComponents() {
        return previousComponents;
    }

    /**
     * return true if the element is a previous element
     * 
     * @param element
     * @return
     */
    public boolean containsComponentInSamePipe(Component element) {
        Iterator it = previousComponents.iterator();
        boolean val = false;
        if (element != this) {
            while (it.hasNext()) {
                Component previous = (Component) it.next();
                val = previous.containsComponentInSamePipe(element);
            }
        } else {
            val = true;
        }
        return val;
    }

    public void isSelected(boolean b) {
        setChanged();
        if (b) {
            notifyObservers(COMPONENT_SELECTED);
        } else
            notifyObservers(COMPONENT_DESELECTED);
    }
}