/*
 * Criado em 28/10/2004
 */
package br.usp.ime.coed.test;

import br.usp.ime.coed.model.CoedDocument;
import br.usp.ime.coed.model.CoedDocumentCareTaker;
import br.usp.ime.coed.model.CoedDocumentMemento;
import junit.framework.TestCase;

/**
 * CoedDocumentCareTaker Unit test
 * 
 * @see br.usp.ime.coed.model.CoedDocumentCareTaker
 * 
 * @author Andr Yoshiaki Kashiwabara <yoshiaki@vision.ime.usp.br>
 */
public class CoedDocumentCareTakerTest extends TestCase {

    private CoedDocumentCareTaker careTaker; 

    /*
     * @see junit.framework.TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
        careTaker = new CoedDocumentCareTaker();
    }
    
    public void testAddMemento() {
        CoedDocument document = new CoedDocument();
        document.setName("TESTE");
        assertTrue(careTaker.undoSize() == 0);
        careTaker.addMemento(document.createMemento());
        assertTrue(careTaker.undoSize() == 1);
        assertTrue(careTaker.redoSize() == 0);
        careTaker.undo(document);
        assertTrue(careTaker.undoSize() == 0);
        assertTrue(careTaker.redoSize() == 1);
        careTaker.addMemento(document.createMemento());
        assertTrue(careTaker.undoSize() == 1);
        assertTrue(careTaker.redoSize() == 0);
    }

    public void testUndo() {
        CoedDocument document = new CoedDocument();
        document.setName("TESTE");
        assertTrue(careTaker.undoSize() == 0);
        careTaker.addMemento(document.createMemento());
        assertTrue(careTaker.undoSize() == 1);
        assertTrue(careTaker.redoSize() == 0);
        CoedDocumentMemento memento = careTaker.undo(document);
        assertTrue(memento.getState().getAllComponents() != document.getAllComponents());
        assertEquals(memento.getState().getName(), document.getName());
        assertTrue(careTaker.undoSize() == 0);
        assertTrue(careTaker.redoSize() == 1);
    }

    public void testRedo() {
        CoedDocument document = new CoedDocument();
        document.setName("TESTE");
        assertTrue(careTaker.undoSize() == 0);
        careTaker.addMemento(document.createMemento());
        assertTrue(careTaker.undoSize() == 1);
        assertTrue(careTaker.redoSize() == 0);
        CoedDocumentMemento memento = careTaker.undo(document);
        CoedDocumentMemento memento2 = careTaker.redo(memento.getState());
        // must be different -- the memento makes a document clone
        assertTrue(memento.getState().getAllComponents() != memento2.getState().getAllComponents());
        assertTrue(careTaker.undoSize() == 1);
        assertTrue(careTaker.redoSize() == 0);
    }

    public void testRedoSize() {
        CoedDocument document = new CoedDocument();
        document.setName("TESTE");
        careTaker.clear();
        assertTrue(careTaker.redoSize() == 0);
        careTaker.addMemento(document.createMemento());
        CoedDocumentMemento memento = careTaker.undo(document);
        assertTrue(careTaker.redoSize() == 1);
    }

    public void testUndoSize() {
        CoedDocument document = new CoedDocument();
        document.setName("TESTE");
        careTaker.clear();
        assertTrue(careTaker.undoSize() == 0);
        careTaker.addMemento(document.createMemento());
        assertTrue(careTaker.undoSize() == 1);
        CoedDocumentMemento memento = careTaker.undo(document);
        assertTrue(careTaker.undoSize() == 0);
        assertTrue(careTaker.redoSize() == 1);
    }

    public void testClear() {
        CoedDocument document = new CoedDocument();
        document.setName("TESTE");
        careTaker.clear();
        assertTrue(careTaker.undoSize() == 0);
        careTaker.addMemento(document.createMemento());
        assertTrue(careTaker.undoSize() == 1);
        CoedDocumentMemento memento = careTaker.undo(document);
        careTaker.clear();
        assertTrue(careTaker.undoSize() == 0);
        assertTrue(careTaker.redoSize() == 0);
    }
}