/*
 * Criado em 28/10/2004
 */
package br.usp.ime.coed.test;

import java.io.File;

import junit.framework.TestCase;
import br.usp.ime.coed.Main;
import br.usp.ime.coed.model.CoedDocument;
import br.usp.ime.coed.model.CoedDocumentManager;

/**
 * @author Andr Yoshiaki Kashiwabara <yoshiaki@vision.ime.usp.br>
 */
public class CoedDocumentManagerTest extends TestCase {

    CoedDocumentManager manager;

    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
        Main m = Main.getInstance();
        m.setEgeneDir(".");
        m.setConfigurationFile("conf/egene.cnf");
        manager = new CoedDocumentManager();
    }

    public void testCloseDocument() {
        try {
            manager.openDocument("test/", "pipe_teste.gen");
        } catch (Exception e) {
            fail();
        }

        assertEquals(manager.getCurrentDocument().getName(), "pipe_teste.gen");
        assertTrue(manager.containsDocument("pipe_teste.gen"));
        manager.closeDocument();
        assertFalse(manager.containsDocument("pipe_teste.gen"));
    }

    public void testCloseDocument2() {
        try {
            int numOpenedDocument = manager.getDocuments().size();
            for (int i = 0; i < numOpenedDocument; i++) {
                manager.closeDocument();
            }
            
            // Close one more
            manager.closeDocument();

            assertTrue(manager.getDocuments().size() == 0);
        } catch (Exception e) {
            fail();
        }
    }

    public void testNewDocument() {
        manager.newDocument("test/", "new.gen");
        assertEquals(manager.getCurrentDocument().getName(), "new.gen");
        assertTrue(manager.containsDocument("new.gen"));
        manager.closeDocument();
        assertFalse(manager.containsDocument("new.gen"));
    }

    public void testOpenDocument() {
        try {
            manager.openDocument("test/", "pipe_teste.gen");
        } catch (Exception e) {
            fail();
        }

        assertEquals(manager.getCurrentDocument().getName(), "pipe_teste.gen");
        assertTrue(manager.containsDocument("pipe_teste.gen"));
        manager.closeDocument();
        assertFalse(manager.containsDocument("pipe_teste.gen"));
    }


    public void testOpenDocumentNotFound() {
        try {
            manager.openDocument("test/", "qiwueqwiouiow.gen");
        } catch (Exception e) {
            assertTrue(e.getMessage().matches(".*No such file or directory.*"));
        }
        try {
            manager.openEGeneDocument("test/", "qiwueqwiouiow.cnf");
        } catch (Exception e) {
            assertTrue(e.getMessage().matches(".*No such file or directory.*"));
        }
   
    }    
    

    public void testOpenEGeneDocument() {
        try {
            manager.openEGeneDocument("test/", "pipe_teste.cnf");
        } catch (Exception e) {
            fail();
        }
        assertEquals(manager.getCurrentDocument().getName(), "pipe_teste.cnf.gen");
        assertTrue(manager.containsDocument("pipe_teste.cnf.gen"));
        manager.closeDocument();
        assertFalse(manager.containsDocument("pipe_teste.cnf.gen"));
    }


    
    public void testSaveAsDocument() {
        try {
            manager.openEGeneDocument("test/", "pipe_teste.cnf");
        } catch (Exception e) {
            fail();
        }
        assertEquals(manager.getCurrentDocument().getName(), "pipe_teste.cnf.gen");
        assertTrue(manager.containsDocument("pipe_teste.cnf.gen"));
        try {
            manager.saveAsDocument("test/", "lixo.gen");
        } catch (Exception e1) {
            e1.printStackTrace();
            fail();
        }
        assertFalse(manager.containsDocument("pipe_teste.cnf.gen"));
        assertTrue(manager.containsDocument("lixo.gen"));
        assertEquals(manager.getCurrentDocument().getName(), "lixo.gen");
        try {
            manager.openEGeneDocument("test/", "lixo.gen");
        } catch (Exception e) {
            fail();
        }
        File f = new File("test/lixo.gen");
        f.delete();
 
    }

    public void testSaveDocument() {
        manager.newDocument("test/", "new.gen");
        assertEquals(manager.getCurrentDocument().getName(), "new.gen");
        assertTrue(manager.containsDocument("new.gen"));
        try {
            manager.saveDocument();
        } catch (Exception e1) {
            e1.printStackTrace();
            fail();
        }
        manager.closeDocument();
        assertFalse(manager.containsDocument("new.gen"));
        try {
            manager.openEGeneDocument("test/", "new.gen");
        } catch (Exception e) {
            fail();
        }
        File f = new File("test/new.gen");
        f.delete();
    }

    public void testSetCurrentDocument() {
        manager.newDocument("test/", "new.gen");
        assertEquals(manager.getCurrentDocument().getName(), "new.gen");
        CoedDocument doc = manager.getCurrentDocument();
        assertTrue(manager.containsDocument("new.gen"));
        try {
            manager.openEGeneDocument("test/", "pipe_teste.cnf");
        } catch (Exception e) {
            fail();
        }
        assertEquals(manager.getCurrentDocument().getName(), "pipe_teste.cnf.gen");
        CoedDocument doc2 = manager.getCurrentDocument();
        assertTrue(manager.containsDocument("pipe_teste.cnf.gen"));

        manager.setCurrentDocument(doc);
        assertTrue(manager.containsDocument("new.gen"));
        manager.closeDocument();
        assertFalse(manager.containsDocument("new.gen"));
        manager.setCurrentDocument(doc2);
        manager.closeDocument();
        assertFalse(manager.containsDocument("pipe_teste.gen"));
             
    }

    public void testGetDocumentCareTaker() {
        manager.newDocument("test/", "new.gen");
        assertEquals(manager.getCurrentDocument().getName(), "new.gen");
        CoedDocument doc = manager.getCurrentDocument();
        assertTrue(manager.containsDocument("new.gen"));
        try {
            manager.openEGeneDocument("test/", "pipe_teste.cnf");
        } catch (Exception e) {
            fail();
        }
        assertEquals(manager.getCurrentDocument().getName(), "pipe_teste.cnf.gen");
        CoedDocument doc2 = manager.getCurrentDocument();
        assertTrue(manager.containsDocument("pipe_teste.cnf.gen"));
        manager.setCurrentDocument(doc);
        assertNotNull(manager.getDocumentCareTaker(doc));
        assertTrue(manager.containsDocument("new.gen"));
        manager.closeDocument();
        assertNull(manager.getDocumentCareTaker(doc));
        assertFalse(manager.containsDocument("new.gen"));
        manager.setCurrentDocument(doc2);
        assertNotNull(manager.getDocumentCareTaker(doc2));
        manager.closeDocument();
        assertNull(manager.getDocumentCareTaker(doc2));
        assertFalse(manager.containsDocument("pipe_teste.gen"));
    }

    public void testContainsDocument() {
        manager.newDocument("test/", "new.gen");
        assertEquals(manager.getCurrentDocument().getName(), "new.gen");
        CoedDocument doc = manager.getCurrentDocument();
        assertTrue(manager.containsDocument("new.gen"));
        try {
            manager.openEGeneDocument("test/", "pipe_teste.cnf");
        } catch (Exception e) {
            fail();
        }
        assertEquals(manager.getCurrentDocument().getName(), "pipe_teste.cnf.gen");
        CoedDocument doc2 = manager.getCurrentDocument();
        assertTrue(manager.containsDocument("pipe_teste.cnf.gen"));
        manager.setCurrentDocument(doc);
        assertTrue(manager.containsDocument("new.gen"));
        manager.closeDocument();
        assertNull(manager.getDocumentCareTaker(doc));
        assertFalse(manager.containsDocument("new.gen"));
        manager.setCurrentDocument(doc2);
        assertNotNull(manager.getDocumentCareTaker(doc2));
        manager.closeDocument();
        assertFalse(manager.containsDocument("pipe_teste.gen"));
    }

    public void testGetDocuments() {
        assertNotNull(manager.getDocuments());
    }

    public void testSaveAsEGeneDocument() {
        try {
            manager.openEGeneDocument("test/", "pipe_teste.cnf");
        } catch (Exception e) {
            fail();
        }
        assertEquals(manager.getCurrentDocument().getName(), "pipe_teste.cnf.gen");
        assertTrue(manager.containsDocument("pipe_teste.cnf.gen"));
        try {
            manager.saveAsEGeneDocument("test/", "lixo.cnf");
        } catch (Exception e1) {
            e1.printStackTrace();
            fail();
        }
        manager.closeDocument();
        
        try {
            manager.openEGeneDocument("test/", "lixo.cnf");
        } catch (Exception e) {
            fail();
        }
        File f = new File("test/lixo.cnf");
        f.delete();
    }

    public void testSaveCurrentDocumentState() {
        try {
            manager.openEGeneDocument("test/", "pipe_teste.cnf");
        } catch (Exception e) {
            fail();
        }
        assertEquals(manager.getCurrentDocument().getName(), "pipe_teste.cnf.gen");
        assertTrue(manager.containsDocument("pipe_teste.cnf.gen"));
        manager.saveCurrentDocumentState();
        assertTrue(manager.getDocumentCareTaker(manager.getCurrentDocument()).undoSize() == 1);
        manager.closeDocument();
    
    }

}