/*
 * Criado em 26/10/2004
 */
package br.usp.ime.coed.view;

import java.util.Iterator;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import br.usp.ime.coed.action.RunPipelineAction;
import br.usp.ime.coed.action.addMenu.AddAction;
import br.usp.ime.coed.action.edit.EditMenuActions;
import br.usp.ime.coed.action.fileMenu.FileMenuActions;
import br.usp.ime.coed.action.pipe.AddPipeAction;
import br.usp.ime.coed.model.CoedDocument;
import br.usp.ime.coed.model.CoedDocumentManager;

/**
 * @author Andr Yoshiaki Kashiwabara <yoshiaki@vision.ime.usp.br>
 */
public class ActionsStateManager implements Observer {
    private EditMenuActions editActions;
    private ApplicationView view;
    private FileMenuActions fileActions;
    private AddPipeAction addPipeAction;
    private List addActions;
    private RunPipelineAction runAction;
    public ActionsStateManager(ApplicationView view) {
        this.view = view;
        this.fileActions = view.getFileMenuActions();
        this.addActions = view.getAddActions();
        this.addPipeAction = view.getAddPipeAction();
        this.editActions = view.getEditMenuActions();
        this.runAction = view.getRunAction();
        addPipeAction.setEnabled(false);
        editActions.setAllEnable(false);
        fileActions.setAllEnable(false);
        fileActions.setInitialActionEnable(true);
        runAction.setEnabled(false);
        setAddActionEnable(false);
    
    }
    /* (non-Javadoc)
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable o, Object arg) {
        if (o instanceof CoedDocument){
            updateCoedDocument(o);
        }else if( o instanceof CoedDocumentManager ){
            CoedDocumentManager manager = (CoedDocumentManager)o;
            if(manager.getDocuments().size() <= 0){
                fileActions.getSaveAsAction().setEnabled(false);
                editActions.setAllEnable(false);
                fileActions.getCloseAction().setEnabled(false);
                fileActions.getExportEGeneAction().setEnabled(false);
                setAddActionEnable(false);
                addPipeAction.setEnabled(false);
                runAction.setEnabled(false);
            }else{
                runAction.setEnabled(true);
                fileActions.getCloseAction().setEnabled(true);
                fileActions.getExportEGeneAction().setEnabled(true);

                fileActions.getSaveAsAction().setEnabled(true);
                editActions.getSelectAll().setEnabled(true);
                setAddActionEnable(true);
                addPipeAction.setEnabled(true);
            }
            updateCoedDocument(manager.getCurrentDocument());
        }
    
    }
    private void updateCoedDocument(Observable o) {
        CoedDocument doc = (CoedDocument)o;
        if(doc == null){
            fileActions.getSaveAction().setEnabled(false);
            return;         
        }
        if(doc.modified()){
            fileActions.getSaveAction().setEnabled(true);
        }else{
            fileActions.getSaveAction().setEnabled(false);
        }
    }
    public void setAllEnable(boolean b) {
        fileActions.setAllEnable(b);
        addPipeAction.setEnabled(b);
        runAction.setEnabled(b);
        setAddActionEnable(b);
    }
    private void setAddActionEnable(boolean b) {
        Iterator it =  addActions.iterator();
        while(it.hasNext()){
            AddAction action = (AddAction)it.next();
            action.setEnabled(b);
        }
    }
    
}