package br.usp.ime.coed.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;

import br.usp.ime.coed.DEFSupport.AbstractDrawingElementFinderClient;
import br.usp.ime.coed.DEFSupport.DrawingElement;
import br.usp.ime.coed.DEFSupport.DrawingElementSelection;
import br.usp.ime.coed.model.CoedDocument;
import br.usp.ime.coed.model.CoedDocumentCareTaker;
import br.usp.ime.coed.model.Component;

/**
 * Extension of <code>AbstractDrawingElementFinderClient</code> class. <BR>
 * This class works with more complicated <code>DrawingElements</code>, like
 * the <code>GraphicalIconElement</code>s.
 */
public class GraphicPane extends AbstractDrawingElementFinderClient implements
        Observer {
    private ApplicationView view;

    private CoedDocument document;

    private Pipe temporaryPipe;

    private ArrayList pipelist = new ArrayList();

    private HashMap componentTOGraphicalIconElement = new HashMap();

    private boolean showPipelineName;

    public GraphicPane(ApplicationView view, CoedDocument document) {
        this.document = document;
        this.view = view;
        document.addObserver(this);
        componentTOGraphicalIconElement = new HashMap();
        setSize(new Dimension(800, 600));
        setPreferredSize(new Dimension(1024, 1024));

    }

    public CoedDocument getDocument() {
        return document;
    }

    public void setDocument(CoedDocument document) {
        this.document.deleteObserver(this);
        this.document = document;
        this.document.addObserver(this);
    }

    /**
     * Paints the pane on <code>g</code>.
     * 
     * @param g
     *            where the panel will be painted on.
     */
    public void paint(Graphics g) {
        super.paint(g);
        if (temporaryPipe != null) {
            temporaryPipe.paintOn((Graphics2D) g);
        }
        Iterator it = pipelist.iterator();
        while (it.hasNext()) {
            Pipe p = (Pipe) it.next();
            p.paintOn((Graphics2D) g);
        }
        // Display pipeline name
        if (!this.getDocument().getPipelineName().equals("")
                && this.showPipelineName) {
            Graphics2D graphics = (Graphics2D) g;
            Font tagFont = new Font("Courier", Font.BOLD, 11);
            FontRenderContext frc = graphics.getFontRenderContext();
            TextLayout layout = new TextLayout("|", tagFont, frc);
            layout = new TextLayout(this.getDocument().getPipelineName(),
                    tagFont, frc);
            Rectangle2D bounds2 = layout.getBounds();
            Rectangle2D bounds1 = (Rectangle2D) bounds2.clone();

            bounds2 = layout.getBounds();
            double height2 = bounds2.getHeight();
            double width2 = bounds2.getWidth();
            bounds2.setRect(5, 5, width2 + 8, height2 + 8);
            bounds1.setRect(4, 4, width2 + 10, height2 + 10);
            graphics.setColor(new Color(0, 0, 0)); /* rectangle color */
            graphics.draw(bounds1);
            graphics.fill(bounds1);

            graphics.setColor(new Color(255, 239, 219)); /* rectangle color */
            graphics.draw(bounds2);
            graphics.fill(bounds2);
            graphics.setColor(new Color(0, 0, 0)); /* font color */
            layout.draw(graphics,
                    (float) (4 + bounds2.getWidth() / 2 - width2 / 2),
                    (float) (bounds2.getY() + height2 + 4));
        }

    }

    public void showPipelineName(boolean b) {
        showPipelineName = b;
        repaint();
    }

    /*
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable o, Object arg) {
        String action = (String) arg;
        if (action == CoedDocument.ADD_COMPONENT) {
            addComponent();
        } else if (action == CoedDocument.ADD_PIPE) {
            addPipe();
        } else if (action == CoedDocument.SET_MEMENTO) {
            setMemento();
        }
        repaint();
    }

    private void setMemento() {
        pipelist.clear();
        deleteElements();
        componentTOGraphicalIconElement.clear();
        Iterator it = document.getAllComponents().iterator();
        // Add all components
        document.setLastComponentAdded(null);
        while (it.hasNext()) {
            Component c = (Component) it.next();
            GraphicalIconElement icon = new GraphicalIconElement(c, this);
            addDrawingElement(icon);
            componentTOGraphicalIconElement.put(c, icon);
        }
        // Add all pipes
        it = componentTOGraphicalIconElement.keySet().iterator();
        while (it.hasNext()) {
            Component from = (Component) it.next();
            Iterator it2 = from.getNextComponents().iterator();
            GraphicalIconElement icon1 = (GraphicalIconElement) componentTOGraphicalIconElement
                    .get(from);
            while (it2.hasNext()) {
                GraphicalIconElement icon2 = (GraphicalIconElement) componentTOGraphicalIconElement
                        .get(it2.next());
                pipelist.add(new Pipe(icon1, icon2, PipeFormat
                        .getDefaultFormat()));
            }
        }
        CoedDocumentCareTaker careTaker = view.getApplicationModel()
                .getDocManager().getDocumentCareTaker(document);
    }

    private void addPipe() {
        GraphicalElement from = (GraphicalElement) componentTOGraphicalIconElement
                .get(document.getLastFrom());
        GraphicalElement to = (GraphicalElement) componentTOGraphicalIconElement
                .get(document.getLastTo());
        pipelist.add(new Pipe(from, to, PipeFormat.getDefaultFormat()));
    }

    private void addComponent() {
        Component c = document.getLastComponentAdded();
        if (c != null) {
            document.setLastComponentAdded(null);
            GraphicalIconElement icon = new GraphicalIconElement(c, this);
            addDrawingElement(icon);
            componentTOGraphicalIconElement.put(c, icon);
        }
    }

    public void setTemporaryPipe(Pipe p) {
        temporaryPipe = p;
        repaint();
    }

    public void selectAll() {
        super.selectAll();
        updateEditActions();

    }

    private void updateEditActions() {
        if (selection.getSelectedElements().size() > 0) {
            view.getEditMenuActions().getClear().setEnabled(true);
            view.getEditMenuActions().getCopy().setEnabled(true);
            view.getEditMenuActions().getCut().setEnabled(true);
        } else {
            view.getEditMenuActions().getClear().setEnabled(false);
            view.getEditMenuActions().getCopy().setEnabled(false);
            view.getEditMenuActions().getCut().setEnabled(false);
        }
    }

    public void selectAll(Set des) {
        super.selectAll(des);
        updateEditActions();
    }

    public void clearSelection() {
        super.clearSelection();
        updateEditActions();
    }

    public void drawingElementClicked(DrawingElement de, MouseEvent e) {
        super.drawingElementClicked(de, e);
        updateEditActions();
        if (e.getButton() == MouseEvent.BUTTON3) {
            if (de instanceof GraphicalIconElement) {
                GraphicalIconElementMenu popupMenu = GraphicalIconElementMenu
                        .getInstance(view, de);
                popupMenu.setAllActionsEnable(true);
                popupMenu.show(e.getComponent(), (int) e.getPoint().getX(),
                        (int) e.getPoint().getY());
                ((GraphicalIconElement) de).getComponent().isSelected(false);
                updateEditActions();
            } else if (de instanceof DrawingElementSelection) {
                DrawingElementSelectionMenu popupMenu = DrawingElementSelectionMenu
                        .getInstance(view, de);
                popupMenu.setAllActionsEnable(true);
                popupMenu.show(e.getComponent(), (int) e.getPoint().getX(),
                        (int) e.getPoint().getY());
                updateEditActions();
            }

        }
    }

    public void drawingElementEndDrag() {
        super.drawingElementEndDrag();
        updateEditActions();
    }

}