/*
 * Criado em 22/10/2004
 */
package br.usp.ime.coed.view;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import br.usp.ime.coed.DEFSupport.DrawingElement;

/**
 * @author Andr? Yoshiaki Kashiwabara <yoshiaki@vision.ime.usp.br>
 */
public class GraphicalElement implements DrawingElement{
    /** Denotes a <code>GraphicalElement side</code> */
    public static int LEFT = 0, RIGHT = 1, TOP = 2, BOTTOM = 3;

    /** Indicates if there is a pipe line drawn in the side specified */
    protected boolean[] sides;

    /** Defines the space ocupied by the <code>GraphicalElement</code> */
    protected Rectangle2D bounds;

    /** Defines the <code>GraphicalElement</code> color */
    protected Color mainColor;

    
    /**
     * Constructor.<BR>
     * Sets the width and height as 50.
     * @param color the element color
     * @param x the x position
     * @param y the y position
     */
    public GraphicalElement(Color color, int x, int y){
      this.mainColor  = color;
      bounds = new Rectangle2D.Float( x, y, 50, 50 );
      sides = new boolean[4];
    }

    /**
     * Constructor.
     * @param color the element color
     * @param x the x position
     * @param y the y position
     * @param width the element width
     * @param height the element height
     */
     public GraphicalElement(Color color, int x, int y, int width, int height) {
      mainColor  = color;
      bounds     = new Rectangle2D.Float( x, y, width, height );
      sides = new boolean[4];
    }

    /**
     * Returns the x position.
     * @return a <b>double</b> value
     */
    public double getX(){
      return bounds.getX();
    }

    /**
     * Returns the y position.
     * @return a <b>double</b> value
     */
    public double getY(){
      return bounds.getY();
    }

    /**
     * Sets the element position.
     * @param x the new x position
     * @param y the new y position
     */
    public void setPosition(double x, double y){
      bounds.setRect( x, y, bounds.getWidth() , bounds.getHeight() );
    }

    /**
     * Returns the rectangular space ocupied by this element.
     * @return a <code>Rectangle2D</code> object
     */
    public Rectangle2D getBounds(){
      return bounds;
    }

    /**
     * Indicates if the space ocupied by this element contains the point
     * <code>p</code>.
     * @param p the point
     * @return a <b>boolean</b>
     */
    public boolean contains(Point2D p) {
      return bounds.contains( p );
    }

    /**
     * Paints the element.
     * @param g where the element is to be painted on
     */
    public void paintOn(Graphics2D g){
      sides[LEFT] = false;
      sides[RIGHT] = false;
      sides[TOP] = false;
      sides[BOTTOM] = false;
    }

    /**
     * Returns the position of the element.
     * @return a <code>Point</code> object
     */
    public Point getPosition(){
      return this.bounds.getBounds().getLocation();

    }

    /**
     * Sets the element width.
     * @param width the new width
     */
    public void setWidth(int width){
      bounds.setRect( getX(), getY(), width , bounds.getHeight() );
    }

    /**
     * Sets the element height
     * @param height the new height
     */
    public void setHeight(int height){
      bounds.setRect( getX(), getY(), bounds.getWidth(), height  );
    }

    /**
     * Sets the color of this element.
     * @param c the new color
     */
    public void setMainColor(Color c){
      this.mainColor = c;
    }

    /**
     * Returns the color of this element.
     * @return a <code>Color</code> object
     * @see Color
     */
    public Color getMainColor(){
      return mainColor;
    }

    /**
     * Returns the center x position of the element.
     * @return an <code>int</code>
     */
    public int getCenterX(){
      return (int)this.bounds.getCenterX();
    }

    /**
     * Return the center y position of the element.
     * @return an <code>int</code>
     */
    public int getCenterY(){
      return (int)this.bounds.getCenterY();
    }

    /**
     * Creates a new Point that refers to the center position of this.
     * @return the center point
     */
    public Point getCenter(){
      return new Point((int)this.bounds.getCenterX(), (int)this.bounds.getCenterY());
    }

    /**
     * Indicates if there is a line drawn at the specified side
     * @param side mut be one of the constants <code>BOTTOM, TOP, LEFT, RIGHT</code>
     * @return a <code>boolean</code> value
     */
    public boolean lineAtSide(int side){
      try{
        return sides[side];
      }catch(IndexOutOfBoundsException e){
        throw new IllegalArgumentException();
      }
    }

    /**
     * Tell the element that there is a line drawn at the specified side.
     * @param side mut be one of the constants <code>BOTTOM, TOP, LEFT, RIGHT</code>
     */
    public void setLineAtSide(int side){
      try{
        sides[side] = true;
      }catch(IndexOutOfBoundsException e){
        throw new IllegalArgumentException();
      }
    }
 }
