package br.usp.ime.coed.view;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.Rectangle2D;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;

import br.usp.ime.coed.model.Component;
import br.usp.ime.coed.model.ComponentType;

/**
 * This class is a <code>GraphicalElement</code> that displays an icon.
 */
public class GraphicalIconElement extends GraphicalElement implements Observer {

    /**
     * Auxiliar variable
     */
    public static int FIRST_HALF = 1, SECOND_HALF = 2;

    /**
     * Model
     */
    private Component component;

    /**
     * A list of graphical pipelines .
     */
    private HashSet pipelist = new HashSet();

    /**
     * the graphicpane
     */
    private GraphicPane pane;

    /**
     * Constructor. <br>
     * If <code>fileName</code> contains a transparent gif, the icon will be
     * transparent too.
     * 
     * @param fileName
     *            the name of the file picture
     * @param x
     *            the x position in which this icon will be drawed
     * @param y
     *            the y position in which this icon will be drawed
     */
    public GraphicalIconElement(int x, int y, ComponentType iconData,
            GraphicPane pane) {
        super(Color.black, x, y);
        component = new Component(iconData);
        component.setX(x);
        component.setY(y);
        setHeight(component.getType().getIcon().getIconHeight());
        setWidth(component.getType().getIcon().getIconWidth());
        component.addObserver(this);
        this.pane = pane;
    }

    /**
     * @param c
     */
    public GraphicalIconElement(Component c, GraphicPane pane) {
        super(Color.black, (int) c.getX(), (int) c.getY());
        component = c;
        setHeight(component.getType().getIcon().getIconHeight());
        setWidth(component.getType().getIcon().getIconWidth());
        component.addObserver(this);
        this.pane = pane;
    }

    /**
     * Paints this element on <code>g</code>.
     * 
     * @param g
     *            where the element will be painted on
     */
    public void paintOn(Graphics2D g) {
        String phaseNameAux = component.getPhase();
        String phase = component.getPhase();
        super.paintOn(g);
        g.setColor(mainColor);
        g.draw(bounds);
        Font tagFont = new Font("Courier", Font.PLAIN, 11);
        component.getType().getIcon().paintIcon(new Canvas(), g,
                new Double(getX()).intValue(), new Double(getY()).intValue());
        if (phase.length() > 11) {
            phaseNameAux = phase.substring(0, 10) + "...";
        }

        FontRenderContext frc = g.getFontRenderContext();
        TextLayout layout = new TextLayout("|", tagFont, frc);
        Rectangle2D bounds2 = layout.getBounds();
        double height1 = bounds2.getHeight();
        layout = new TextLayout(phaseNameAux, tagFont, frc);
        bounds2 = layout.getBounds();
        double height2 = bounds2.getHeight();
        double width2 = bounds2.getWidth();
        bounds2.setRect((double) (getX()), (double) (getY() - 1.75 * height1),
                bounds.getWidth(), 1.5 * height1);
        g.draw(bounds2);
        if(pane.getDocument().getFirstComponent() == this.getComponent()){
            g.setColor(new Color(200, 50, 50)); /* rectangle color */
        }else{
            g.setColor(new Color(255, 239, 219)); /* rectangle color */
        }

        g.fill(bounds2);
        if(pane.getDocument().getFirstComponent() == this.getComponent()){
        	g.setColor(new Color(255, 255, 255)); /* font color */
        }else{
        	g.setColor(new Color(58, 95, 205)); /* font color */
        }
        layout.draw(g,
                (float) (getX() + 0.5 * (bounds.getWidth()) - 0.5 * (width2)),
                (float) (getY() - 0.65 * height1));
        g.setColor(mainColor);
        Iterator it = pipelist.iterator();

    }

    /**
     * Sets the element position.
     * 
     * @param x
     *            the new x position
     * @param y
     *            the new y position
     */
    public void setPosition(double x, double y) {
        super.setPosition(component.getX(), component.getY());
        component.setPosition(x, y);
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable o, Object arg) {
        super.setPosition(component.getX(), component.getY());
        if (arg == Component.COMPONENT_SELECTED) {
           pane.getSelection().addToSelection(this);
        } else if (arg == Component.COMPONENT_DESELECTED) {
            pane.getSelection().removeFromSelection(this);
        }
        pane.repaint();
    }

    public Component getComponent() {
        return component;
    }

    public void setComponent(Component component) {
        this.component = component;
    }

    public HashSet getPipelist() {
        return pipelist;
    }

    public void setPipelist(HashSet pipelist) {
        this.pipelist = pipelist;
    }

    
}

