package br.usp.ime.coed.view;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import br.usp.ime.coed.ReadEgeneConfiguration;
import br.usp.ime.coed.action.FormattedAction;
import br.usp.ime.coed.model.ApplicationModel;
import br.usp.ime.coed.model.CoedDocumentCareTaker;
import br.usp.ime.coed.model.ComponentType;

/**
 * The form that requests the icon creation parameter values.
 */
public class IconValuesForm extends JFrame {

    /** Form panel button */
    private JButton ok, cancel;

    /** The the optional parameters fields */
    private JTextField[] optionalFields;

    /** The the mandatory parameters fields */
    private JTextField[] mandatoryFields;

    /** The mandatory fields of the <code>GraphicalIconElement</code> */
    private String[] mandatoryFieldNames;

    /** The optional fields of the <code>GraphicalIconElement</code> */
    private String[] optionalFieldNames;

    /** The phase name field */
    private JTextField phaseNameField;

    private JPopupMenu popUpMenu;

    private ComponentType type;

    private GraphicalIconElement icon;

    private GraphicPane pane;

    private ApplicationModel model;

    private ApplicationView view;

    private boolean beginEnable;

    public IconValuesForm(GraphicalIconElement icon, ApplicationView view) {
        super("Edit the " + icon.getComponent().getType().getName()
                + " component");
        this.type = icon.getComponent().getType();
        this.icon = icon;
        this.pane = view.getTabPane().getSelectedGraphicPane();
        this.view = view;
        this.model = view.getApplicationModel();
        beginEnable = true;
        formatWindow();
    }

    /**
     * Creates the text fields, buttons, etc. Makes the form.
     */
    private void formatWindow() {
        JMenu findFileMenu = new JMenu();
        FormattedAction findFileAction = new FormattedAction("Find File", 'f') {
            public void actionPerformed(ActionEvent e) {
                Component comp = getFocusOwner();
                JTextField field;

                if (comp != null) {
                    try {
                        field = (JTextField) comp;
                        JFileChooser chooser = new JFileChooser();
                        chooser
                                .setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
                        int returnValue = chooser.showDialog(field, "Paste");
                        if (returnValue == JFileChooser.APPROVE_OPTION)
                            field.setText(chooser.getSelectedFile()
                                    .getAbsolutePath());
                    } catch (ClassCastException ce) {
                    }
                }

            }
        };
        this.popUpMenu = new JPopupMenu();
        findFileAction.addTo(popUpMenu);

        Box formBox;
        /* READING THE ICON PARAMETERS */
        ReadEgeneConfiguration conf = ReadEgeneConfiguration.getInstance();
        Map mandatoryParameters = conf.getMandatoryParameters(type.getName());
        Map optionalParameters = conf.getOptionalParameters(type.getName());

        int numberOfParameters = mandatoryParameters.size()
                + optionalParameters.size();
        /* CREATING THE FORM BOX */
        formBox = makeForm(icon, mandatoryParameters, optionalParameters);

        /* CREATING THE BUTTON PANEL */
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout(FlowLayout.TRAILING));

        createButtons();
        if (mandatoryParameters.size() == 0)
            ok.setEnabled(true);
 
        
        ok.setMnemonic('k');
        cancel.setMnemonic('c');
        buttonPanel.add(ok);
        buttonPanel.add(cancel);

        /* ADDING THE BOXES TO THIS WINDOW */
        Container container = this.getContentPane();
        container.setLayout(new BorderLayout());

        Dimension screen = getToolkit().getScreenSize();

        Box mainBox = Box.createVerticalBox();
        JScrollPane scroll = new JScrollPane(formBox);
        scroll.setBorder(null);

        mainBox.add(scroll);
        mainBox.add(buttonPanel);

        container.add(addMargin(mainBox, 10));

        /* FINALIZING ... */
        pack();

        centralizeOnScreen(500, Math.min(formBox.getHeight() + 90, (int) screen
                .getHeight() - 25));
        setResizable(false);
        setVisible(true);

    }

    /**
     * Auxiliary method that centrelizes the window on the screen
     * 
     * @param width
     *            the window width
     * @param height
     *            the window height
     */
    private void centralizeOnScreen(int width, int height) {

        Dimension screen = getToolkit().getScreenSize();

        setBounds((int) (screen.getWidth() - width) / 2, (int) (screen
                .getHeight() - height) / 2, width, height);
    }

    /**
     * Auxiliary method that create the buttons (ok, cancel).
     */
    private void createButtons() {

        ok = new JButton("OK");
        if(beginEnable){
            ok.setEnabled(true);
        }else{
            ok.setEnabled(false);
        }
        ok.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {

                /* reading mandatory values from mandatory fields */
                Map mandatoryValues = new HashMap();
                for (int i = 0; i < mandatoryFields.length; i++)
                    mandatoryValues.put(mandatoryFieldNames[i],
                            mandatoryFields[i].getText());
                /* reading optional values from optional fields */
                Map optionalValues = new HashMap();
                for (int i = 0; i < optionalFields.length; i++)
                    optionalValues.put(optionalFieldNames[i], optionalFields[i]
                            .getText());

                String phaseName = phaseNameField.getText();
                icon.getComponent().setMandatoryValues(mandatoryValues);
                icon.getComponent().setOptionalValues(optionalValues);
                icon.getComponent().setPhase(phaseName);
                /* omitting form */
                setVisible(false);
                CoedDocumentCareTaker caretaker = model.getDocManager()
                        .getDocumentCareTaker(pane.getDocument());
                caretaker.addMemento(pane.getDocument().createMemento());
                if (!pane.getDocument().getAllComponents().contains(
                        icon.getComponent()))
                    pane.getDocument().addComponent(icon.getComponent());
                view.setAllActionEnable(true);
                view.getApplicationModel().getDocManager().updateObserver();

            }
        });

        cancel = new JButton("Cancel");
        cancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                view.setAllActionEnable(true);
                view.getApplicationModel().getDocManager().updateObserver();
                setVisible(false);
            }
        });

    }

    /**
     * Makes the form <code>Box</code>
     * 
     * @param mandatoryParameters
     *            the mandatory values to request
     * @param optionalParameters
     *            the optional values to request
     * @return a <code>Box</code>
     */
    private Box makeForm(GraphicalIconElement icon, Map mandatoryParameters,
            Map optionalParameters) {

        Box formBox = Box.createVerticalBox();
        Box mandatoryBox = Box.createVerticalBox();
        Box optionalBox = Box.createVerticalBox();

        /* MANDATORY FIELDS */

        Set questions = mandatoryParameters.keySet();
        Iterator iterator = questions.iterator();
        phaseNameField = new JTextField(icon.getComponent().getPhase());
        phaseNameField.getDocument().addDocumentListener(new FieldListener());
        mandatoryBox.add(new JLabel("PHASE:"));
        mandatoryBox.add(phaseNameField);

        mandatoryFields = new JTextField[questions.size()];
        mandatoryFieldNames = new String[questions.size()];

        for (int i = 0; i < mandatoryFields.length; i++) {
            mandatoryFieldNames[i] = (String) iterator.next();
            JLabel label = new JLabel(mandatoryFieldNames[i] + ":");
            mandatoryBox.add(label);
            JTextField field = new JTextField((String) mandatoryParameters
                    .get(mandatoryFieldNames[i]));
            String aux = (String) icon.getComponent().getMandatoryValues()
            .get(mandatoryFieldNames[i]);
            field.setText(aux);
            if(aux == null || aux.trim().equals(""))
                beginEnable = false;
            
            field.getDocument().addDocumentListener(new FieldListener());
            field.add(popUpMenu);
            field.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    if (e.getButton() == MouseEvent.BUTTON3) {
                        popUpMenu.show(e.getComponent(), e.getX(), e.getY());
                        e.getComponent().requestFocus();
                    }
                }
            });
            mandatoryBox.add(field);
            mandatoryFields[i] = field;
            mandatoryBox.add(Box.createVerticalStrut(5));
        }

        mandatoryBox = addMargin(mandatoryBox, 10);
        mandatoryBox.setBorder(new TitledBorder("Mandatory fields"));

        /* OPTIONAL */

        questions = optionalParameters.keySet();
        iterator = questions.iterator();
        optionalFields = new JTextField[questions.size()];
        optionalFieldNames = new String[questions.size()];

        for (int i = 0; i < optionalFields.length; i++) {
            optionalFieldNames[i] = (String) iterator.next();
            JLabel label = new JLabel(optionalFieldNames[i] + ":");
            optionalBox.add(label);
            JTextField field = new JTextField((String) optionalParameters
                    .get(optionalFieldNames[i]));
            if (icon.getComponent().getOptionalValues().containsKey(
                    optionalFieldNames[i])) {
                field.setText((String) icon.getComponent().getOptionalValues()
                        .get(optionalFieldNames[i]));
            }
            field.add(popUpMenu);
            field.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    if (e.getButton() == MouseEvent.BUTTON3) {
                        popUpMenu.show(e.getComponent(), e.getX(), e.getY());
                        e.getComponent().requestFocus();
                    }
                }
            });
            optionalBox.add(field);
            optionalFields[i] = field;
            optionalBox.add(Box.createVerticalStrut(5));
        }

        optionalBox = addMargin(optionalBox, 10);
        optionalBox.setBorder(new TitledBorder("Optional fields"));

        /* FORM */
        if (mandatoryFields.length != 0 && optionalFields.length != 0) {
            formBox.add(mandatoryBox);
            formBox.add(Box.createVerticalStrut(5));
            formBox.add(optionalBox);
        } else if (mandatoryFields.length != 0 && optionalFields.length == 0)
            formBox.add(mandatoryBox);
        else if (mandatoryFields.length == 0 && optionalFields.length != 0) {
            formBox.add(mandatoryBox);
            formBox.add(optionalBox);
        }

        else
            // there are no values to request
            formBox = null;

        return formBox;
    }

    /**
     * Auxiliary method that adds a margin to a box
     * 
     * @param box
     *            the <code>Box</code>
     * @param margin
     *            the margin width in pixels
     * @return a <code>Box</code>
     */
    private Box addMargin(Box box, int margin) {

        Box dummyVertical = Box.createVerticalBox();
        dummyVertical.add(Box.createVerticalStrut(margin));
        dummyVertical.add(box);
        dummyVertical.add(Box.createVerticalStrut(margin));

        Box dummyHorizontal = Box.createHorizontalBox();
        dummyHorizontal.add(Box.createHorizontalStrut(margin));
        dummyHorizontal.add(dummyVertical);
        dummyHorizontal.add(Box.createHorizontalStrut(margin));

        return dummyHorizontal;
    }

    /**
     * 
     * <p>
     * The listener that enables or disables the ok button according to the
     * mandatory fields values.
     * </p>
     */
    private class FieldListener implements DocumentListener {

        /**
         * Interface method implementation.
         * 
         * @param e
         *            a <code>DocumentEvent</code>
         */
        public void changedUpdate(DocumentEvent e) {
            changed();
        }

        /**
         * Interface method implementation.
         * 
         * @param e
         *            a <code>DocumentEvent</code>
         */
        public void insertUpdate(DocumentEvent e) {
            changed();
        }

        /**
         * Interface method implementation.
         * 
         * @param e
         *            a <code>DocumentEvent</code>
         */
        public void removeUpdate(DocumentEvent e) {
            changed();
        }

        /**
         * Notificates that a text field value changed.
         */
        private void changed() {
            /* Verify if the mandatory blanks are filled in */
            for (int i = 0; i < mandatoryFields.length; i++)
                if (mandatoryFields[i].getText().trim().equals("")) {
                    ok.setEnabled(false);
                    return;
                }

            if (phaseNameField.getText().trim().equals("")) {
                ok.setEnabled(false);
                return;
            }

            ok.setEnabled(true);
        }
    } // FieldListener

} // IconValuesForm
