package br.usp.ime.coed.view;

import java.awt.Color;

import br.usp.ime.coed.ProgramColors;

/**
 * A pipe format provider. Facilitates a pipe construction.
 * <br>A pipe may be constructed without this class, even though.
 */
public class PipeFormat {

  /** The pipe line color */
  private Color lineColor;

  /** The pipe arrow measure */

  private int arrowWidth, arrowLength;

  /** Indicates if the pipe is dashed */
  private boolean dashed;

  /** Standard pipe format. */
  private static PipeFormat
                defaultFormat = new PipeFormat(ProgramColors.getPipeColor(),4, 6, false),
                animationFormat =new PipeFormat(ProgramColors.getAnimationColor(), 4, 6, true);
  /**
   * Default pipe format provider.
   * @return <code>defaultFormat</code>
   */
  public static PipeFormat getDefaultFormat(){
    return defaultFormat;
  }

  /**
   * Animation pipe format provider.
   * @return <code>animationFormat</code>
   */
  public static PipeFormat getAnimationFormat(){
    return animationFormat;
  }

  /**
   * Constructor.
   * @param lineColor the pipe line color
   * @param arrowWidth the pipe arrow width
   * @param arrowLength the pipe arrow length
   * @param dashed indicates if the pipe is dashed
   */
  public PipeFormat(Color lineColor, int arrowWidth, int arrowLength, boolean dashed){
    this.lineColor = lineColor;
    this.arrowWidth = arrowWidth;
    this.arrowLength = arrowLength;
    this.dashed = dashed;
  }

  /**
   * Returns the pipe line color.
   * @return a <code>Color</code>
   */
  public Color getLineColor(){
    return lineColor;
  }

  /**
   * Returns the pipe arrow width.
   * @return a <code>int</code> value
   */
  public int getArrowWidth(){
    return arrowWidth;
  }

  /**
   * Returns the pipe arrow length.
   * @return a <code>int</code> value
   */
  public int getArrowLength(){
    return arrowLength;
  }

  /**
   * Indicates if the pipe is dashed.
   * @return a <code>boolean</code>
   */
  public boolean isDashed(){
    return dashed;
  }

}
